
#pragma once

#include <boost/mpl/eval_if.hpp>
#include <boost/mpl/placeholders.hpp>

#include "../gen_pwm_args_type.hpp"
#include "gen_process_statement.hpp"

namespace ketchup { namespace detail {

///////////////////////////////////////////////////////////////////////////////
// gen_message_processor_base
//

struct base_for_cracked_handlers_aux
{
	bool b_msg_handled;
	bool is_msg_handled() { return b_msg_handled; }
	void set_msg_handled(bool b_) { b_msg_handled = b_; }	
};

template< class Derived, class HasCrackedHandlers >
struct gen_message_processor_base
{
	typedef typename gen_pwm_args_type<Derived>::type arguments_type;
	typedef typename gen_pwm_args_reference<Derived>::type arguments_reference;

	struct base
	{
		template< class MessageMap >
		BOOL process_window_message(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT& lResult, DWORD dwMsgMapID)
		{
			Derived& derived = *static_cast<Derived*>(this);
			BOOL bHandled = TRUE;
			arguments_type xs(derived, hWnd, uMsg, wParam, lParam, lResult, dwMsgMapID, bHandled);

			typedef typename gen_process_statement<MessageMap, arguments_reference>::type statement;
			return statement::process(xs) ? TRUE : FALSE;
		};

		typedef base type;
	};	

	struct base_for_cracked_handlers : base_for_cracked_handlers_aux
	{
		template< class MessageMap >
		BOOL process_window_message(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT& lResult, DWORD dwMsgMapID)
		{
			Derived& derived = *static_cast<Derived*>(this);
			BOOL bHandled = TRUE;
			arguments_type xs(derived, hWnd, uMsg, wParam, lParam, lResult, dwMsgMapID, bHandled);

			bool old = derived.is_msg_handled();	// for recursive calls
			typedef typename gen_process_statement<MessageMap, arguments_reference>::type statement;
			BOOL ret = statement::process(xs) ? TRUE : FALSE;
   			derived.set_msg_handled(old);
			return ret;
		};

		typedef base_for_cracked_handlers type;
	};

	typedef typename boost::mpl::eval_if<HasCrackedHandlers,
		base_for_cracked_handlers,
		base
	>::type type;
};

} } // namespace ketchup::detail
