
#pragma once

#include "../gen_pwm_args_type.hpp"
#include "../cmd_ui.hpp"
#include "../update_cmd_ui.hpp"

namespace ketchup { namespace handler {

template< class Derived >
struct cmd_ui_handlers
{
	typedef typename gen_pwm_args_reference<Derived>::type pwm_args_ref;
	///////////////////////////////////////////////////////////////////////////////
	// cmd_ui_handler
	//
	template< UINT nID, void (Derived::*func)(cmd_ui&) >
	struct cmd_ui_handler
	{
		static bool process(pwm_args_ref xs)
		{
			if (get_uMsg<Derived>(xs) == update_cmd_ui_message_id())
			{
				cmd_ui& ui = get_cmd_ui_from_wParam(get_wParam<Derived>(xs));
				if (ui.get_id() == nID) {
					get_bHandled<Derived>(xs) = TRUE;
					(get_derived<Derived>(xs).*func)(ui);
					if (get_bHandled<Derived>(xs)) {
						get_lResult<Derived>(xs) = TRUE;	// That's it!
						return true;											// make lResult alive!
					}
				}
			}
			return false;
		}
	};

	///////////////////////////////////////////////////////////////////////////////
	// chain_client_cmd_ui
	//
	struct chain_client_cmd_ui
	{
		static bool process(pwm_args_ref xs)
		{
			if (get_uMsg<Derived>(xs) == update_cmd_ui_message_id())
			{
				HWND hWnd = get_derived<Derived>(xs).m_hWndClient;
				if (hWnd != NULL) {
					BOOL bHandled = (BOOL)::SendMessage(hWnd, get_uMsg<Derived>(xs), get_wParam<Derived>(xs), get_lParam<Derived>(xs));
          get_lResult<Derived>(xs) = bHandled;
					return bHandled == TRUE;
				}
			}
			return false;
		}
	};
	///////////////////////////////////////////////////////////////////////////////
	// chain_client_cmd_ui
	//
	struct chain_mdi_child_cmd_ui
	{
		static bool process(pwm_args_ref xs)
		{
			if (get_uMsg<Derived>(xs) == update_cmd_ui_message_id())
			{
				HWND hWnd = get_derived<Derived>(xs).MDIGetActive();
				if (hWnd != NULL) {
					BOOL bHandled = (BOOL)::SendMessage(hWnd, get_uMsg<Derived>(xs), get_wParam<Derived>(xs), get_lParam<Derived>(xs));
          get_lResult<Derived>(xs) = bHandled;
					return bHandled == TRUE;
				} 
			}
			return false;
		}
	};

	///////////////////////////////////////////////////////////////////////////////
	// menu_cmd_ui_generator
	//
	struct menu_cmd_ui_generator
	{
		static bool process(pwm_args_ref xs)
		{
			if (get_uMsg<Derived>(xs) == WM_INITMENUPOPUP)
			{
				update_menu_cmd_ui(get_hWnd<Derived>(xs), (HMENU)get_wParam<Derived>(xs));
			}
			return false;
		}
	};

	///////////////////////////////////////////////////////////////////////////////
	// chain_msg_map_cmd_ui_updater
	//
	template< class EnableIDs, class Entry >
	struct cmd_ui_handler_auto_enable
	{
		struct fold_init
		{
			static bool includes(UINT) { return false; }
		};

		template< class State, class nID >
		struct fold_op
		{
			static bool includes(UINT cmd_ui_id)
			{
				if (cmd_ui_id == nID::value)
					return true;
				else
					return State::includes(cmd_ui_id);
			}
		};

		static bool process(pwm_args_ref xs)
		{
			typedef typename boost::mpl::fold<EnableIDs,
				fold_init,
				fold_op<boost::mpl::_1, boost::mpl::_2>
			>::type IDs_;

			if (get_uMsg<Derived>(xs) == update_cmd_ui_message_id())
			{
				bool bHandled = Entry::process(xs); // cmd_ui handled!
				cmd_ui& cmd_ui = get_cmd_ui_from_wParam(get_wParam<Derived>(xs));
				if (IDs_::includes(cmd_ui.get_id())) {
					cmd_ui.enable(bHandled);
				}
				return bHandled;
			}

			return false;
		}
	};
};

} } // namespace ketchup::handler
